"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.patchOne = void 0;
const tslib_1 = require("tslib");
const common_1 = require("@nestjs/common");
const typeorm_1 = require("typeorm");
const helper_1 = require("../../../../../helper");
function patchOne(options) {
    var e_1, _a;
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const startTime = Date.now();
        const preparedResourceName = (0, helper_1.snakeToCamel)(this.repository.metadata.name);
        const { route: { id: id }, } = options;
        const body = options.body;
        const { id: idBody, attributes, relationships } = body;
        if (`${id}` !== idBody) {
            throw new common_1.UnprocessableEntityException({
                source: { pointer: '/data/id' },
                detail: "Data 'id' must be equal to url param",
            });
        }
        const whereCondition = {
            id: (0, typeorm_1.Equal)(id),
        };
        const target = yield this.repository.findOne({
            where: whereCondition,
        });
        if (!target) {
            throw new common_1.NotFoundException({
                detail: `Resource '${preparedResourceName}' with id '${id}' does not exist`,
            });
        }
        const prepareParams = Date.now() - startTime;
        for (const attr in attributes) {
            target[attr] = attributes[attr];
        }
        const relationshipsArray = [];
        if (relationships) {
            try {
                for (var _b = tslib_1.__asyncValues(this.UtilsMethode.asyncIterateFindRelationships(relationships, this.repository)), _c; _c = yield _b.next(), !_c.done;) {
                    const relationshipsTarget = _c.value;
                    relationshipsArray.push(relationshipsTarget.propsName);
                    target[relationshipsTarget.propsName] = relationshipsTarget.rel;
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) yield _a.call(_b);
                }
                finally { if (e_1) throw e_1.error; }
            }
        }
        const callQuery = Date.now() - startTime;
        return this.repository.save(target).then((r) => {
            const returnData = Object.keys(r).reduce((acum, item) => {
                if (!relationshipsArray.includes(item)) {
                    acum[item] = r[item];
                }
                return acum;
            }, {});
            const resultTransform = this.transform.transformData(returnData, []);
            const transform = Date.now() - startTime;
            const debug = {
                prepareParams,
                callQuery: callQuery - prepareParams,
                transform: transform - callQuery,
            };
            return {
                meta: Object.assign({}, (this.config.debug ? { debug } : {})),
                data: resultTransform,
            };
        });
    });
}
exports.patchOne = patchOne;
//# sourceMappingURL=patch-one.js.map